package eu.mathiasl.legacychatfix;

import com.comphenix.protocol.PacketType;
import com.comphenix.protocol.events.ListenerPriority;
import com.comphenix.protocol.events.PacketAdapter;
import com.comphenix.protocol.events.PacketEvent;
import com.comphenix.protocol.reflect.StructureModifier;
import com.comphenix.protocol.wrappers.WrappedChatComponent;

import eu.mathiasl.legacychatfix.config.ConfigManager;
import eu.mathiasl.legacychatfix.config.LegacyChatOption;

import org.bukkit.configuration.ConfigurationSection;
import org.bukkit.configuration.MemorySection;
import org.bukkit.configuration.file.FileConfiguration;
import org.bukkit.plugin.Plugin;

import java.util.HashMap;
import java.util.Set;

public class LegacyChatFixPacketAdapter extends PacketAdapter {

    /**
     * Legacy color codes to replace.
     * TODO: Add support for custom legacy color codes through
     *  the config file.
     */
    private HashMap<String, LegacyChatOption> legacyColors;
    /*= new HashMap<String, String>() {
        {
            put("\\<black>", "black");
            put("\\<dark_blue>", "dark_blue");
            put("\\<dark_green>", "dark_green");
            put("\\<dark_aqua>", "dark_aqua");
            put("\\<dark_red>", "dark_red");
            put("\\<dark_purple>", "dark_purple");
            put("\\<gold>", "gold");
            put("\\<gray>", "gray");
            put("\\<dark_gray>", "dark_gray");
            put("\\<blue>", "blue");
            put("\\<green>", "green");
            put("\\<aqua>", "aqua");
            put("\\<red>", "red");
            put("\\<light_purple>", "light_purple");
            put("\\<yellow>", "yellow");
            put("\\<white>", "white");
            put("\\<bold>", "bold");
            put("\\<strikethrough>", "strikethrough");
            put("\\<underline>", "underline");
            put("\\<italic>", "italic");
            put("\\<magic>", "obfuscated");
            put("\\<secondary>", "gray");
        }
    };*/

    /**
     * Set the adapter to listen for packets being sent to the client from "system chat".
     *
     * @param plugin A reference to the plugin class responsible for the adapter.
     */
    public LegacyChatFixPacketAdapter(Plugin plugin, ConfigManager configManager) {
        super(plugin, ListenerPriority.NORMAL, PacketType.Play.Server.SYSTEM_CHAT);
        this.legacyColors = getLegacyChatOptions(configManager);
    }

    /**
     * Get the legacy chat options from the config file.
     *
     * @param configManager The config manager to get the config file from.
     * @return A map of legacy color codes to replace.
     */
    private HashMap<String, LegacyChatOption> getLegacyChatOptions(ConfigManager configManager) {
        HashMap<String, LegacyChatOption> legacyColors = new HashMap<>();

        FileConfiguration config = configManager.getConfig("config.yml");
        ConfigurationSection legacyColorsSection = config.getConfigurationSection("legacy_codes");

        if (legacyColorsSection == null) return null;
        Set<String> legacyColorsKeys = legacyColorsSection.getKeys(false);

        legacyColorsSection.getValues(false).forEach((key, value) -> {
            if (!(value instanceof MemorySection)) return;

            MemorySection valueSection = (MemorySection) value;
            String legacyCode = valueSection.getString("legacy_code");
            String modernCode = valueSection.getString("modern_code");

            try {
                LegacyChatOption legacyChatOption = new LegacyChatOption(legacyCode, modernCode);
                legacyColors.put(key, legacyChatOption);
            } catch (Exception e) {
                plugin.getLogger().warning("Failed to parse legacy color option: legacy_code=" + legacyCode + ", modern_code=" + modernCode);
            }
        });

        return legacyColors;
    }

    /**
     * Listen for packets being sent to the client and replace legacy color codes.
     *
     * @param event The packet event being sent.
     */
    @Override
    public void onPacketSending(PacketEvent event) {
        // Get the chat components from the packet
        StructureModifier<WrappedChatComponent> chatComponents = event.getPacket().getChatComponents();
        // If there are no chat components, return
        if (chatComponents == null || chatComponents.size() == 0) return;

        // Get the chat component
        WrappedChatComponent chatComponent = chatComponents.read(0); // Get the chat component
        // Convert the chat component to a JSON string
        String message = chatComponent.getJson(); // Get the message

        boolean containedLegacyCoding = false;
        for (String legacyColorIdentifier : legacyColors.keySet()) {
            LegacyChatOption legacyColorOption = legacyColors.get(legacyColorIdentifier);
            // If the text does not contain the legacy color code, continue
            if (!message.contains(legacyColorOption.getLegacyCode()))
                continue;

            containedLegacyCoding = true;
            // Replace the legacy color code with an empty string
            message = message.replace(legacyColorOption.getLegacyCode().replace("\\", "\\\\"), legacyColorOption.getModernCode().toString());
        }

        // No reason to continue if the message doesn't contain any legacy color codes
        if (!containedLegacyCoding) return;
        // Write the modified chat components back to the packet
        try {
            chatComponents.write(0, WrappedChatComponent.fromJson(message));
        } catch (Exception e) {
            plugin.getLogger().warning("Failed to write chat components back to the packet: " + e.getCause());
        }
    }
}
